@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema;
 use vars '$VERSION';
-$VERSION = '1.32';
+$VERSION = '1.33';
 
 use base 'XML::Compile';
 
@@ -98,13 +98,12 @@ sub addSchemas($@)
     {   push @nsopts, $o => delete $opts{$o} if exists $opts{$o};
     }
 
-
     UNIVERSAL::isa($node, __PACKAGE__)
         and error __x"use useSchema(), not addSchemas() for a {got} object"
              , got => ref $node;
 
     UNIVERSAL::isa($node, 'XML::LibXML::Node')
-        or error __x"required is a XML::LibXML::Node";
+        or error __x"addSchema() requires an XML::LibXML::Node";
 
     $node = $node->documentElement
         if $node->isa('XML::LibXML::Document');
@@ -192,12 +191,14 @@ sub compile($$@)
         exists $args{check_occurs}   or $args{check_occurs} = 1;
     }
 
-    my $iut = exists $args{ignore_unused_tags} ? $args{ignore_unused_tags}
-      : $self->{unused_tags};
+    my $iut = exists $args{ignore_unused_tags}
+      ? $args{ignore_unused_tags} : $self->{unused_tags};
+
     $args{ignore_unused_tags}
       = !defined $iut ? undef : ref $iut eq 'Regexp' ? $iut : qr/^/;
 
-    exists $args{include_namespaces} or $args{include_namespaces} = 1;
+    exists $args{include_namespaces}
+        or $args{include_namespaces} = 1;
 
     if($args{sloppy_integers} ||= 0)
     {   eval "require Math::BigInt";
@@ -325,11 +326,12 @@ sub template($@)
     my %show = map {("show_$_" => 1)} split m/\,/, $show;
     my $nss  = $self->namespaces;
 
-    my $indent              = $args{indent} || "  ";
-    $args{check_occurs}     = 1;
-    $args{mixed_elements} ||= 'ATTRIBUTES';
-    $args{default_values} ||= 'EXTEND';
-    $args{abstract_types} ||= 'ERROR';
+    my $indent                  = $args{indent} || "  ";
+    $args{check_occurs}         = 1;
+    $args{mixed_elements}     ||= 'ATTRIBUTES';
+    $args{default_values}     ||= 'EXTEND';
+    $args{abstract_types}     ||= 'ERROR';
+    $args{elements_qualified} ||= 'TOP';
 
     exists $args{include_namespaces}
         or $args{include_namespaces} = 1;
@@ -398,8 +400,8 @@ sub namespaces() { shift->{namespaces} }
 my (%schemaByFilestamp, %schemaByChecksum);
 
 sub importDefinitions($@)
-{   my ($self, $thing, %options) = @_;
-    my @data = ref $thing eq 'ARRAY' ? @$thing : $thing;
+{   my ($self, $frags, %options) = @_;
+    my @data = ref $frags eq 'ARRAY' ? @$frags : $frags;
 
     # this is a horrible hack, but by far the simpelest solution to
     # avoid dataToXML process the same info twice.
